#!perl

# Simple Perl Script to illustrate use of WMI to gather system information
# and display it in the same format at the Windows Management Console
#
# Use Win32_VideoController for Display Adapter information

use Win32::OLE qw(in with);
use Win32::Registry;
use Date::Calc;

use constant ReturnImmediately => 0x10;
use constant ForwardOnly       => 0x20;

# Pick a host that you have the necessary rights to monitor

my $host = "localhost";

# Connect to WMI service

my $WMI = Win32::OLE->GetObject("winmgmts:\\\\$host\\root\\CIMV2") || die "WMI connection failed.\n";

# Gather Video Controller Information

my $set = $WMI->ExecQuery("SELECT * FROM Win32_VideoController", "WQL", ReturnImmediately | ForwardOnly);

my %VideoMemoryType = ( '1'  => 'Other',
			'2'  => 'Unknown',
			'3'  => 'VRAM',
			'4'  => 'DRAM',
			'5'  => 'SRAM',
			'6'  => 'WRAM',
			'7'  => 'EDO RAM',
			'8'  => 'Burst Synchronous DRAM',
			'9'  => 'Pipelined Burst SRAM',
			'10' => 'CDRAM',
			'11' => '3DRAM',
			'12' => 'SDRAM',
			'13' => 'SGRAM',
		      );

my $i = 0;
foreach my $obj (in $set)
{
	my ($size, $unit) = ($obj->{AdapterRAM}, 'B');
	   ($size, $unit) = normalize_filesize($size, $unit);
	    $size = sprintf("%.2f", $size);

	print "                      WMI: Win32_VideoController $i\n";
	print "------------------------------------------------------------------\n";
	print "                  Caption: ". $obj->{Caption}                   ."\n";
	print "               AdapterRAM: ". $obj->{AdapterRAM}                ." B\t($size $unit)\n";
	print "              Description: ". $obj->{Description}               ."\n";
	print "  InstalledDisplayDrivers: ". $obj->{InstalledDisplayDrivers}   ."\n";
	print "               DriverDate: ". $obj->{DriverDate}                ."\n";
	print "            DriverVersion: ". $obj->{DriverVersion}             ."\n";
	print "          VideoMemoryType: ". $obj->{VideoMemoryType}           ."\t\t($VideoMemoryType{$obj->{VideoMemoryType}})\n";
	print "     VideoModeDescription: ". $obj->{VideoModeDescription}      ."\n";
	print "           VideoProcessor: ". $obj->{VideoProcessor}            ."\n";
	print "\n";
}

# Gather Operating System Information

$set = $WMI->ExecQuery("SELECT * FROM Win32_OperatingSystem", "WQL", ReturnImmediately | ForwardOnly);

my %OperatingSystemSKU = ( '0'  => 'Undefined',
			   '1'  => 'Ultimate Edition',
			   '2'  => 'Home Basic Edition',
			   '3'  => 'Home Premium Edition',
			   '4'  => 'Enterprise Edition',
			   '5'  => 'Home Basic N Edition',
			   '6'  => 'Business Edition',
			   '7'  => 'Standard Server Edition',
			   '8'  => 'Datacenter Server Edition',
			   '9'  => 'Small Business Server Edition',
			   '10' => 'Enterprise Server Edition',
			   '11' => 'Starter Edition',
			   '12' => 'Datacenter Server Core Edition',
			   '13' => 'Standard Server Core Edition',
			   '14' => 'Enterprise Server Core Edition',
			   '15' => 'Enterprise Server Edition for Itanium-Based Systems',
			   '16' => 'Business N Edition',
			   '17' => 'Web Server Edition',
			   '18' => 'Cluster Server Edition',
			   '19' => 'Home Server Edition',
			   '20' => 'Storage Express Server Edition',
			   '21' => 'Storage Standard Server Edition',
			   '22' => 'Storage Workgroup Server Edition',
			   '23' => 'Storage Enterprise Server Edition',
			   '24' => 'Server For Small Business Edition',
			   '25' => 'Small Business Server Premium Edition',
			 );

my %OSType = (  '0'  => 'Unknown',
		'1'  => 'Other',
		'2'  => 'MACROS',
		'3'  => 'ATTUNIX',
		'4'  => 'DGUX',
		'5'  => 'DECNT',
		'6'  => 'Digital UNIX',
		'7'  => 'OpenVMS',
		'8'  => 'HPUX',
		'9'  => 'AIX',
		'10' => 'MVS',
		'11' => 'OS400',
		'12' => 'OS/2',
		'13' => 'JavaVM',
		'14' => 'MSDOS',
		'15' => 'WIN3x',
		'16' => 'WIN95',
		'17' => 'WIN98',
		'18' => 'WINNT',
		'19' => 'WINCE',
		'20' => 'NCR3000',
		'21' => 'NetWare',
		'22' => 'OSF',
		'23' => 'DC/OS',
		'24' => 'Reliant UNIX',
		'25' => 'SCO UnixWare',
		'26' => 'SCO OpenServer',
		'27' => 'Sequent',
		'28' => 'IRIX',
		'29' => 'Solaris',
		'30' => 'SunOS',
		'31' => 'U6000',
		'32' => 'ASERIES',
		'33' => 'TandemNSK',
		'34' => 'TandemNT',
		'35' => 'BS2000',
		'36' => 'LINUX',
		'37' => 'Lynx',
		'38' => 'XENIX',
		'39' => 'VM/ESA',
		'40' => 'Interactive UNIX',
		'41' => 'BSDUNIX',
		'42' => 'FreeBSD',
		'43' => 'NetBSD',
		'44' => 'GNU Hurd',
		'45' => 'OS9',
		'46' => 'MACH Kernel',
		'47' => 'Inferno',
		'48' => 'QNX',
		'49' => 'EPOC',
		'50' => 'IxWorks',
		'51' => 'VxWorks',
		'52' => 'MiNT',
		'53' => 'BeOS',
		'54' => 'HP MPE',
		'55' => 'NextStep',
		'56' => 'PalmPilot',
		'57' => 'Rhapsody',
	     );

my %ProductType = ( '1' => 'Work Station',
		    '2' => 'Domain Controller',
		    '3' => 'Server',
		  );

$i = 0;
foreach my $obj (in $set)
{
	my ($size, $unit) = ($obj->{FreePhysicalMemory}, 'KB');
	   ($size, $unit) = normalize_filesize($size, $unit);
	    $size = sprintf("%.2f", $size);

	my ($size2, $unit2) = ($obj->{FreeVirtualMemory}, 'KB');
	   ($size2, $unit2) = normalize_filesize($size2, $unit2);
	    $size2 = sprintf("%.2f", $size2);

	my ($size3, $unit3) = ($obj->{TotalVirtualMemorySize}, 'KB');
	   ($size3, $unit3) = normalize_filesize($size3, $unit3);
	    $size3 = sprintf("%.2f", $size3);

	my ($size4, $unit4) = ($obj->{TotalVisibleMemorySize}, 'KB');
	   ($size4, $unit4) = normalize_filesize($size4, $unit4);
	    $size4 = sprintf("%.2f", $size4);

	my $date_boot = $obj->{LastBootUpTime};
	my $date_now  = $obj->{LocalDateTime};

	# 20080716122003.085000+060
	if($date_boot =~ /^(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})\.(\d{6})(\+|\-)(\d{3})$/) {
		my $year        = $1;
		my $month       = $2;
		my $day         = $3;
		my $hour        = $4;
		my $minute      = $5;
		my $second      = $6;
		my $millisecond = $7;
		my $sign        = $8;
		my $tz_min_diff = $9;
		
		$date_boot = "$year-$month-$day $hour:$minute:$second";
	}

	if($date_now =~ /^(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})\.(\d{6})(\+|\-)(\d{3})$/) {
		my $year        = $1;
		my $month       = $2;
		my $day         = $3;
		my $hour        = $4;
		my $minute      = $5;
		my $second      = $6;
		my $millisecond = $7;
		my $sign        = $8;
		my $tz_min_diff = $9;
		
		$date_now = "$year-$month-$day $hour:$minute:$second";
	}

	my @boot = split /-|:| /, $date_boot;
	my @now  = split /-|:| /, $date_now;

	my $year1  = $boot[0];
	my $month1 = $boot[1];
	my $day1   = $boot[2];
	my $hour1  = $boot[3];
	my $min1   = $boot[4];
	my $sec1   = $boot[5];

	my $year2  = $now[0];
	my $month2 = $now[1];
	my $day2   = $now[2];
	my $hour2  = $now[3];
	my $min2   = $now[4];
	my $sec2   = $now[5];

	my ($D_y,$D_m,$D_d, $Dh,$Dm,$Ds) = Date::Calc::Delta_YMDHMS($year1,$month1,$day1, $hour1,$min1,$sec1,
					                	    $year2,$month2,$day2, $hour2,$min2,$sec2);

	my $uptime = "";
	if($D_y > 0) {
		$uptime .= ", " if($uptime ne "");
		$uptime .= "$D_y years";
	}
	if($D_m > 0) {
		$uptime .= ", " if($uptime ne "");
		$uptime .= "$D_m months";
	}
	if($D_d > 0) {
		$uptime .= ", " if($uptime ne "");
		$uptime .= "$D_d days";
	}
	if($Dh > 0) {
		$uptime .= ", " if($uptime ne "");
	   	$uptime .= "$Dh hours";
	}
	if($Dm > 0) {
		$uptime .= ", " if($uptime ne "");
	   	$uptime .= "$Dm minutes";
	}
	if($Ds  > 0) {
		$uptime .= ", " if($uptime ne "");
	   	$uptime .= "$Ds seconds";
	}

	print "                      WMI: Win32_OperatingSystem $i\n";
	print "------------------------------------------------------------------\n";
	print "                  Caption: ". $obj->{Caption}                   ."\n";
	print "                  Version: ". $obj->{Version}                   ."\n";
	print "              BuildNumber: ". $obj->{BuildNumber}               ."\n";
	print "                BuildType: ". $obj->{BuildType}                 ."\n";
	print "       OperatingSystemSKU: ". $obj->{OperatingSystemSKU}        ."\t\t(". $OperatingSystemSKU{$obj->{OperatingSystemSKU}}.")\n";
	print "               CSDVersion: ". $obj->{CSDVersion}                ."\n";
	print "  ServicePackMajorVersion: ". $obj->{ServicePackMajorVersion}   ."\n";
	print "  ServicePackMinorVersion: ". $obj->{ServicePackMinorVersion}   ."\n";
	print "                   CSName: ". $obj->{CSName}                    ."\n";
	print "     OtherTypeDescription: ". $obj->{OtherTypeDescription}      ."\n";
	print "           OSArchitecture: ". $obj->{OSArchitecture}            ."\n";
	print "                   OSType: ". $obj->{OSType}                    ."\t\t(". $OSType{$obj->{OSType}} .")\n";
	print "              ProductType: ". $obj->{ProductType}               ."\t\t(". $ProductType{$obj->{ProductType}} .")\n";
	print "       FreePhysicalMemory: ". $obj->{FreePhysicalMemory}        ." KB\t($size $unit)\n";
	print "   TotalVisibleMemorySize: ". $obj->{TotalVisibleMemorySize}    ." KB\t($size4 $unit4)\n";
	print "        FreeVirtualMemory: ". $obj->{FreeVirtualMemory}         ." KB\t($size2 $unit2)\n";
	print "   TotalVirtualMemorySize: ". $obj->{TotalVirtualMemorySize}    ." KB\t($size3 $unit3)\n";
	print "           LastBootUpTime: ". $obj->{LastBootUpTime}            ."\n";
	print "            LocalDateTime: ". $obj->{LocalDateTime}             ."\n";
	print "                   Uptime: ". $uptime                           ."\n";
	print "\n";
}

# Gather Processor Information

$set = $WMI->ExecQuery("SELECT * FROM Win32_Processor", "WQL", ReturnImmediately | ForwardOnly);

my %Architecture = ( '0' => 'x86',
		     '1' => 'MIPS',
		     '2' => 'Alpha',
		     '3' => 'PowerPC',
		     '6' => 'Intel Itanium Processor Family (IPF)',
		     '9' => 'x64',
		   );

my %Family = ( '1'   => 'Other',
	       '2'   => 'Unknown',
	       '3'   => '8086',
	       '4'   => '80286',
	       '5'   => 'Intel386(TM) Processor',
	       '6'   => 'Intel486(TM) Processor',
	       '7'   => '8087',
	       '8'   => '80287',
	       '9'   => '80387',
	       '10'  => '80487',
	       '11'  => 'Pentium Brand',
	       '12'  => 'Pentium Pro',
	       '13'  => 'Pentium II',
	       '14'  => 'Pentium Processor with MMX(TM) Technology',
	       '15'  => 'Celeron(TM)',
	       '16'  => 'Pentium II Xeon(TM)',
	       '17'  => 'Pentium III',
	       '18'  => 'M1 Family',
	       '19'  => 'M2 Family',
	       '24'  => 'AMD Duron(TM) Processor Family',
	       '25'  => 'K5 Family',
	       '26'  => 'K6 Family',
	       '27'  => 'K6-2',
	       '28'  => 'K6-3',
	       '29'  => 'AMD Athlon(TM) Processor Family',
	       '30'  => 'AMD2900 Family',
	       '31'  => 'K6-2+',
	       '32'  => 'Power PC Family',
	       '33'  => 'Power PC 601',
	       '34'  => 'Power PC 603',
	       '35'  => 'Power PC 603+',
	       '36'  => 'Power PC 604',
	       '37'  => 'Power PC 620',
	       '38'  => 'Power PC X704',
	       '39'  => 'Power PC 750',
	       '48'  => 'Alpha Family',
	       '49'  => 'Alpha 21064',
	       '50'  => 'Alpha 21066',
	       '51'  => 'Alpha 21164',
	       '52'  => 'Alpha 21164PC',
	       '53'  => 'Alpha 21164a',
	       '54'  => 'Alpha 21264',
	       '55'  => 'Alpha 21364',
	       '64'  => 'MIPS Family',
	       '65'  => 'MIPS R4000',
	       '66'  => 'MIPS R4200',
	       '67'  => 'MIPS R4400',
	       '68'  => 'MIPS R4600',
	       '69'  => 'MIPS R10000',
	       '80'  => 'SPARC Family',
	       '81'  => 'SuperSPARC',
	       '82'  => 'microSPARC II',
	       '83'  => 'microSPARC IIep',
	       '84'  => 'UltraSPARC',
	       '85'  => 'UltraSPARC II',
	       '86'  => 'UltraSPARC IIi',
	       '87'  => 'UltraSPARC III',
	       '88'  => 'UltraSPARC IIIi',
	       '96'  => '68040',
	       '97'  => '68xxx Family',
	       '98'  => '68000',
	       '99'  => '68010',
	       '100' => '68020',
	       '101' => '68030',
	       '112' => 'Hobbit Family',
	       '120' => 'Crusoe(TM) TM5000 Family',
	       '121' => 'Crusoe(TM) TM3000 Family',
	       '122' => 'Efficeon(TM) TM8000 Family',
	       '128' => 'Weitek',
	       '130' => 'Itanium(TM) Processor',
	       '131' => 'AMD Athlon(TM) 64 Processor Famiily',
	       '132' => 'AMD Opteron(TM) Processor Family',
	       '144' => 'PA-RISC Family',
	       '145' => 'PA-RISC 8500',
	       '146' => 'PA-RISC 8000',
	       '147' => 'PA-RISC 7300LC',
	       '148' => 'PA-RISC 7200',
	       '149' => 'PA-RISC 7100LC',
	       '150' => 'PA-RISC 7100',
	       '160' => 'V30 Family',
	       '176' => 'Pentium III Xeon(TM) Processor',
	       '177' => 'Pentium III Processor with Intel SpeedStep(TM) Technology',
	       '178' => 'Pentium 4',
	       '179' => 'Intel Xeon(TM)',
	       '180' => 'AS400 Family',
	       '181' => 'Intel Xeon(TM) Processor MP',
	       '182' => 'AMD Athlon(TM) XP Family',
	       '183' => 'AMD Athlon(TM) MP Family',
	       '184' => 'Intel Itanium 2',
	       '185' => 'Intel Pentium M Processor',
	       '190' => 'K7',
	       '200' => 'IBM390 Family',
	       '201' => 'G4',
	       '202' => 'G5',
	       '203' => 'G6',
	       '204' => 'z/Architecture Base',
	       '250' => 'i860',
	       '251' => 'i960',
	       '260' => 'SH-3',
	       '261' => 'SH-4',
	       '280' => 'ARM',
	       '281' => 'StrongARM',
	       '300' => '6x86',
	       '301' => 'MediaGX',
	       '302' => 'MII',
	       '320' => 'WinChip',
	       '350' => 'DSP',
	       '500' => 'Video Processor',
	     );

my %ProcessorType = ( '1' => 'Other',
		      '2' => 'Unknown',
		      '3' => 'Central Processor',
		      '4' => 'Math Processor',
		      '5' => 'DSP Processor',
		      '6' => 'Video Processor',
		    );

my %UpgradeMethod = ( '1'  => 'Other',
		      '2'  => 'Unknown',
		      '3'  => 'Daughter Board',
		      '4'  => 'ZIF Socket',
		      '5'  => 'Replacement or Piggy Back',
		      '6'  => 'None',
		      '7'  => 'LIF Socket',
		      '8'  => 'Slot 1',
		      '9'  => 'Slot 2',
		      '10' => '370 Pin Socket',
		      '11' => 'Slot A',
		      '12' => 'Slot M',
		      '13' => 'Socket 423',
		      '14' => 'Socket A (Socket 462)',
		      '15' => 'Socket 478',
		      '16' => 'Socket 754',
		      '17' => 'Socket 940',
		      '18' => 'Socket 939',
		    );

my %VoltageCaps = ( '1' => '5 volts',
		    '2' => '3.3 volts',
		    '4' => '2.9 volts',
		  );

$i = 0;
foreach my $obj (in $set)
{
	my ($size, $unit) = ($obj->{L2CacheSize}, 'KB');
	   ($size, $unit) = normalize_filesize($size, $unit);
	    $size = sprintf("%.2f", $size);

	my ($size, $unit) = ($obj->{L3CacheSize}, 'KB');
	   ($size, $unit) = normalize_filesize($size, $unit);
	    $size = sprintf("%.2f", $size);

	print "                      WMI: Win32_Processor $i\n";
	print "------------------------------------------------------------------\n";
	print "                  Caption: ". $obj->{Caption}                   ."\n";
	print "             Architecture: ". $obj->{Architecture}              ."\t\t(". $Architecture{$obj->{Architecture}} .")\n";
	print "        CurrentClockSpeed: ". $obj->{CurrentClockSpeed}         ." MHz\n";
	print "           CurrentVoltage: ". $obj->{CurrentVoltage}            ." 1/10 volts\n";
	print "                DataWidth: ". $obj->{DataWidth}                 ."\n";
	print "              Description: ". $obj->{Description}               ."\n";
	print "                 ExtClock: ". $obj->{ExtClock}                  ." MHz\n";
	print "                   Family: ". $obj->{Family}                    ."\t\t(". $Family{$obj->{Family}} .")\n";
	print "              L2CacheSize: ". $obj->{L2CacheSize}               ." KB\t($size $unit)\n";
	print "             L2CacheSpeed: ". $obj->{L2CacheSpeed}              ." MHz\n";
	print "              L3CacheSize: ". $obj->{L3CacheSize}               ." KB\t($size2 $unit2)\n";
	print "             L3CacheSpeed: ". $obj->{L3CacheSpeed}              ." MHz\n";
  	print "                    Level: ". $obj->{Level}                     ."\n";
  	print "           LoadPercentage: ". $obj->{LoadPercentage}            ." %\n";
  	print "             Manufacturer: ". $obj->{Manufacturer}              ."\n";
  	print "            MaxClockSpeed: ". $obj->{MaxClockSpeed}             ." MHz\n";
  	print "                     Name: ". $obj->{Name}                      ."\n";
  	print "            NumberOfCores: ". $obj->{NumberOfCores}             ."\n";
  	print "NumberOfLogicalProcessors: ". $obj->{NumberOfLogicalProcessors} ."\n";
  	print "              ProcessorId: ". $obj->{ProcessorId}               ."\n";
  	print "            ProcessorType: ". $obj->{ProcessorType}             ."\t\t(". $ProcessorType{$obj->{ProcessorType}} .")\n";
  	print "                 Revision: ". $obj->{Revision}                  ."\n";
  	print "                     Role: ". $obj->{Role}                      ."\n";
  	print "        SocketDesignation: ". $obj->{SocketDesignation}         ."\n";
  	print "                 Stepping: ". $obj->{Stepping}                  ."\n";
  	print "            UpgradeMethod: ". $obj->{UpgradeMethod}             ."\t\t(". $UpgradeMethod{$obj->{UpgradeMethod}} .")\n";
  	print "                 UniqueId: ". $obj->{UniqueId}                  ."\n";
  	print "                  Version: ". $obj->{Version}                   ."\n";
  	print "              VoltageCaps: ". $obj->{VoltageCaps}               ."\t\t(". $VoltageCaps{$obj->{VoltageCaps}} .")\n";
	print "\n";
}

# Gather Fan Information

$set = $WMI->ExecQuery("SELECT * FROM Win32_Fan", "WQL", ReturnImmediately | ForwardOnly);

my %Availability = ( '1'  => 'Other',
		     '2'  => 'Unknown',
		     '3'  => 'Running or Full Power',
		     '4'  => 'Warning',
		     '5'  => 'In Test',
		     '6'  => 'Not Applicable',
		     '7'  => 'Power Off',
		     '8'  => 'Off Line',
		     '9'  => 'Off Duty',
		     '10' => 'Degraded',
		     '11' => 'Not Installed',
		     '12' => 'Install Error',
		     '13' => 'Power Save - Unknown',
		     '14' => 'Power Save - Low Power Mode',
		     '15' => 'Power Save - Standby',
		     '16' => 'Power Cycle',
		     '17' => 'Power Save - Warning',
		   );

$i = 0;
foreach my $obj (in $set)
{
	print "                      WMI: Win32_Fan $i\n";
	print "------------------------------------------------------------------\n";
	print "            ActiveCooling: ". $obj->{ActiveCooling}             ."\n";
	print "                  Caption: ". $obj->{Caption}                   ."\n";
	print "              Description: ". $obj->{Description}               ."\n";
	print "             Availability: ". $obj->{Availability}              ."\t\t(". $Availability{$obj->{Availability}} .")\n";
	print "             DesiredSpeed: ". $obj->{DesiredSpeed}              ." RPM\n";
	print "                     Name: ". $obj->{Name}                      ."\n";
	print "            VariableSpeed: ". $obj->{VariableSpeed}             ."\n";
	print "\n";
}

# Gather BIOS Information

$set = $WMI->ExecQuery("SELECT * FROM Win32_BIOS", "WQL", ReturnImmediately | ForwardOnly);

my %BiosCharacteristics = ( '0'  => 'Reserved',
			    '1'  => 'Reserved',
			    '2'  => 'Unknown',
			    '3'  => 'BIOS Characteristics Not Supported',
			    '4'  => 'ISA is supported',
			    '5'  => 'MCA is supported',
			    '6'  => 'EISA is supported',
			    '7'  => 'PCI is supported',
			    '8'  => 'PC Card (PCMCIA) is supported',
			    '9'  => 'Plug and Play is supported',
			    '10' => 'APM is supported',
			    '11' => 'BIOS is Upgradable (Flash)',
			    '12' => 'BIOS shadowing is allowed',
			    '13' => 'VL-VESA is supported',
			    '14' => 'ESCD support is available',
			    '15' => 'Boot from CD is supported',
			    '16' => 'Selectable Boot is supported',
			    '17' => 'BIOS ROM is socketed',
			    '18' => 'Boot From PC Card (PCMCIA) is supported',
			    '19' => 'EDD (Enhanced Disk Drive) Specification is supported',
			    '20' => 'Int 13h - Japanese Floppy for NEC 9800 1.2mb (3.5, 1k Bytes/Sector, 360 RPM) is supported',
			    '21' => 'Int 13h - Japanese Floppy for Toshiba 1.2mb (3.5, 360 RPM) is supported',
			    '22' => 'Int 13h - 5.25 / 360 KB Floppy Services are supported',
			    '23' => 'Int 13h - 5.25 /1.2MB Floppy Services are supported',
			    '24' => 'Int 13h - 3.5 / 720 KB Floppy Services are supported',
			    '25' => 'Int 13h - 3.5 / 2.88 MB Floppy Services are supported',
			    '26' => 'Int 5h, Print Screen Service is supported',
			    '27' => 'Int 9h, 8042 Keyboard services are supported',
			    '28' => 'Int 14h, Serial Services are supported',
			    '29' => 'Int 17h, printer services are supported',
			    '30' => 'Int 10h, CGA/Mono Video Services are supported',
			    '31' => 'NEC PC-98',
			    '32' => 'ACPI is supported',
			    '33' => 'USB Legacy is supported',
			    '34' => 'AGP is supported',
			    '35' => 'I2O boot is supported',
			    '36' => 'LS-120 boot is supported',
			    '37' => 'ATAPI ZIP Drive boot is supported',
			    '38' => '1394 boot is supported',
			    '39' => 'Smart Battery is supported',
			    '40' => 'Reserved for BIOS vendor',
			    '41' => 'Reserved for BIOS vendor',
			    '42' => 'Reserved for BIOS vendor',
			    '43' => 'Reserved for BIOS vendor',
			    '44' => 'Reserved for BIOS vendor',
			    '45' => 'Reserved for BIOS vendor',
			    '46' => 'Reserved for BIOS vendor',
			    '47' => 'Reserved for BIOS vendor',
			    '48' => 'Reserved for system vendor',
			    '49' => 'Reserved for system vendor',
			    '50' => 'Reserved for system vendor',
			    '51' => 'Reserved for system vendor',
			    '52' => 'Reserved for system vendor',
			    '53' => 'Reserved for system vendor',
			    '54' => 'Reserved for system vendor',
			    '55' => 'Reserved for system vendor',
			    '56' => 'Reserved for system vendor',
			    '57' => 'Reserved for system vendor',
			    '58' => 'Reserved for system vendor',
			    '59' => 'Reserved for system vendor',
			    '60' => 'Reserved for system vendor',
			    '61' => 'Reserved for system vendor',
			    '62' => 'Reserved for system vendor',
			    '63' => 'Reserved for system vendor',
			  );

my %TargetOperatingSystem = ( '0'  => 'Unknown',
			      '1'  => 'Other',
			      '2'  => 'MACOS',
			      '3'  => 'ATTUNIX',
			      '4'  => 'DGUX',
			      '5'  => 'DECNT',
			      '6'  => 'Digital Unix',
			      '7'  => 'OpenVMS',
			      '8'  => 'HPUX',
			      '9'  => 'AIX',
			      '10' => 'MVS',
			      '11' => 'OS400',
			      '12' => 'OS/2',
			      '13' => 'JavaVM',
			      '14' => 'MSDOS',
			      '15' => 'WIN3x',
			      '16' => 'WIN95',
			      '17' => 'WIN98',
			      '18' => 'WINNT',
			      '19' => 'WINCE',
			      '20' => 'NCR3000',
			      '21' => 'NetWare',
			      '22' => 'OSF',
			      '23' => 'DC/OS',
			      '24' => 'Reliant UNIX',
			      '25' => 'SCO UnixWare',
			      '26' => 'SCO OpenServer',
			      '27' => 'Sequent',
			      '28' => 'IRIX',
			      '29' => 'Solaris',
			      '30' => 'SunOS',
			      '31' => 'U6000',
			      '32' => 'ASERIES',
			      '33' => 'TandemNSK',
			      '34' => 'TandemNT',
			      '35' => 'BS2000',
			      '36' => 'LINUX',
			      '37' => 'Lynx',
			      '38' => 'XENIX',
			      '39' => 'VM/ESA',
			      '40' => 'Interactive UNIX',
			      '41' => 'BSDUNIX',
			      '42' => 'FreeBSD',
			      '43' => 'NetBSD',
			      '44' => 'GNU Hurd',
			      '45' => 'OS9',
			      '46' => 'MACH Kernel',
			      '47' => 'Inferno',
			      '48' => 'QNX',
			      '49' => 'EPOC',
			      '50' => 'IxWorks',
			      '51' => 'VxWorks',
			      '52' => 'MiNT',
			      '53' => 'BeOS',
			      '54' => 'HP MPE',
			      '55' => 'NextStep',
			      '56' => 'PalmPilot',
			      '57' => 'Rhapsody',
			      '58' => 'Windows 2000',
			      '59' => 'Dedicated',
			      '60' => 'VSE',
			      '61' => 'TPF',
			    );

$i = 0;
foreach my $obj (in $set)
{
	print "                      WMI: Win32_BIOS $i\n";
	print "------------------------------------------------------------------\n";
	foreach my $bios_characteristics (@{$obj->{BiosCharacteristics}}) {
		print "      BiosCharacteristics: ". $bios_characteristics             ."\t\t(". $BiosCharacteristics{$bios_characteristics} .")\n";
	}
	foreach my $bios_version (@{$obj->{BIOSVersion}}) {
		print "              BIOSVersion: ". $bios_version                     ."\n";
	}
	print "              BuildNumber: ". $obj->{BuildNumber}               ."\n";
	print "                  Caption: ". $obj->{Caption}                   ."\n";
	print "                  CodeSet: ". $obj->{CodeSet}                   ."\n";
	print "              Description: ". $obj->{Description}               ."\n";
	print "       IdentificationCode: ". $obj->{IdentificationCode}        ."\n";
	print "             Manufacturer: ". $obj->{Manufacturer}              ."\n";
	print "                     Name: ". $obj->{Name}                      ."\n";
	print "            OtherTargetOS: ". $obj->{OtherTargetOS}             ."\n";
	print "              ReleaseDate: ". $obj->{ReleaseDate}               ."\n";
	print "    TargetOperatingSystem: ". $obj->{TargetOperatingSystem}     ."\t\t(". $TargetOperatingSystem{$obj->{TargetOperatingSystem}} .")\n";
	print "                  Version: ". $obj->{Version}                   ."\n";
	print "\n";
}

# Gather ComputerSystem Information

$set = $WMI->ExecQuery("SELECT * FROM Win32_ComputerSystem", "WQL", ReturnImmediately | ForwardOnly);

my %DomainRole = ( '0' => 'Standalone Workstation',
		   '1' => 'Member Workstation',
		   '2' => 'Standalone Server',
		   '3' => 'Member Server',
		   '4' => 'Backup Domain Controller',
		   '5' => 'Primary Domain Controller',
		 );

my %PCSystemType = ( '0' => 'Unspecified',
		     '1' => 'Desktop',
		     '2' => 'Mobile',
		     '3' => 'Workstation',
		     '4' => 'Enterprise Server',
		     '5' => 'Small Office and Home Office (SOHO) Server',
		     '6' => 'Appliance PC',
		     '7' => 'Performance Server',
		     '8' => 'Maximum',
		   );

my %ThermalState = ( '1' => 'Other',
		     '2' => 'Unknown',
		     '3' => 'Safe',
		     '4' => 'Warning',
		     '5' => 'Critical',
		     '6' => 'Nonrecoverable',
		   );

$i = 0;
foreach my $obj (in $set)
{
	my ($size, $unit) = ($obj->{TotalPhysicalMemory}, 'B');
	   ($size, $unit) = normalize_filesize($size, $unit);
	    $size = sprintf("%.2f", $size);

	print "                      WMI: Win32_ComputerSystem $i\n";
	print "------------------------------------------------------------------\n";
	print "                  Caption: ". $obj->{Caption}                   ."\n";
	print "              Description: ". $obj->{Description}               ."\n";
	print "              DNSHostName: ". $obj->{DNSHostName}               ."\n";
	print "                   Domain: ". $obj->{Domain}                    ."\n";
	print "               DomainRole: ". $obj->{DomainRole}                ."\t\t(". $DomainRole{$obj->{DomainRole}} .")\n";
	print "             Manufacturer: ". $obj->{Manufacturer}              ."\n";
	print "                    Model: ". $obj->{Model}                     ."\n";
	print "                     Name: ". $obj->{Name}                      ."\n";
	print "NumberOfLogicalProcessors: ". $obj->{NumberOfLogicalProcessors} ."\n";
	print "       NumberOfProcessors: ". $obj->{NumberOfProcessors}        ."\n";
	foreach my $oem_string (@{$obj->{OEMStringArray}}) {
		print "                OEMString: ". $oem_string                       ."\n";
	}
	print "             PartOfDomain: ". $obj->{PartOfDomain}              ."\n";
	print "             PCSystemType: ". $obj->{PCSystemType}              ."\t\t(". $PCSystemType{$obj->{PCSystemType}} .")\n";
	print "               SystemType: ". $obj->{SystemType}                ."\n";
	print "             ThermalState: ". $obj->{ThermalState}              ."\t\t(". $ThermalState{$obj->{ThermalState}} .")\n";
	print "      TotalPhysicalMemory: ". $obj->{TotalPhysicalMemory}       ." B\t($size $unit)\n";
	print "\n";
}


################################################################################
# Subroutines

sub normalize_filesize {
        my $size = $_[0] =~ /^\d+$/          ? $_[0] : 0;
        my $unit = $_[1] =~ /^[KMGTPEZY]?B$/ ? $_[1] : 'B';

        my $u     = 0;
        my @units = qw/B KB MB GB TB PB EB ZB YB/;
        foreach(@units) {
                if($unit =~ /^$_/i) {
                        if($size >= 1024) {
                                $size /= 1024;
                                $unit  = $units[$u+1];
                        }
                        else {
                                last;
                        }
                }
                $u++;
        }

        return ($size, $unit);
}

